import numpy as np
from xarray import DataArray


def make_raster(values, attrs=None, coords=None):
    values = _ensure_band_dimension(values)
    coords = coords or dict(band=np.arange(values.shape[0]) + 1,
                            y=np.arange(values.shape[1]),
                            x=np.arange(values.shape[2]),
                            spatial_ref=0)
    array = DataArray(values, dims=['band', 'y', 'x'], coords=coords, attrs=attrs)
    return array


def _ensure_band_dimension(expected):
    expected = np.array(expected)
    if expected.ndim == 2:
        expected = expected[np.newaxis, ...]
    return expected



from pathlib import Path
from typing import Sequence, Optional

import xarray as xr

from pytest_approvaltests_geo.differs.difference import DiffType, Difference, calculate_pixel_diff_stats, \
    add_common_meta_data_diffs
from pytest_approvaltests_geo.float_utils import Tolerance
from pytest_approvaltests_geo.scrubbers import RecursiveScrubber, identity_recursive_scrubber, scrub_xarray_data


class DifferOfGeoZarrs:
    def __init__(self, recursive_scrubber: Optional[RecursiveScrubber] = None,
                 float_tolerance: Optional[Tolerance] = None):
        self._recursive_scrubber = recursive_scrubber or identity_recursive_scrubber
        self._float_tolerance = float_tolerance or Tolerance()

    def diffs(self, received_path: Path, approved_path: Path) -> Sequence[Difference]:
        diffs = []
        with xr.open_zarr(received_path) as received_ds, xr.open_zarr(approved_path) as approved_ds:
            received_ds = scrub_xarray_data(received_ds, self._recursive_scrubber)
            approved_ds = scrub_xarray_data(approved_ds, self._recursive_scrubber)
            diffs = add_common_meta_data_diffs(received_ds, approved_ds, diffs)

            try:
                xr.testing.assert_allclose(received_ds, approved_ds, **self._float_tolerance.to_kwargs())
            except AssertionError as assertion_diff:
                common_data_vars = set(received_ds.data_vars) & set(approved_ds.data_vars)
                stats_per_data_var = [(name, calculate_pixel_diff_stats(received_ds[name], approved_ds[name]))
                                      for name in common_data_vars]
                stats_per_data_var = [(n, s) for n, s in stats_per_data_var if not s.is_empty]
                diff_stats = '\n'.join([f"{n}: {str(s)}" for n, s in stats_per_data_var])
                if diff_stats:
                    diffs.append(Difference(diff_stats, DiffType.PIXEL_STATS))
                diffs.append(Difference(str(assertion_diff), DiffType.DATASET))

        return diffs



from pathlib import Path

from approvaltests.core import Comparator

from pytest_approvaltests_geo.differs.differ_of_geo_tiffs import DifferOfGeoTiffs


class CompareGeoTiffs(Comparator, DifferOfGeoTiffs):

    def compare(self, received_path: str, approved_path: str) -> bool:
        received_path = Path(received_path)
        approved_path = Path(approved_path)
        if not received_path.exists() or not approved_path.exists():
            return False

        diffs = self.diffs(received_path, approved_path)
        return len(diffs) == 0



import shutil
from pathlib import Path

from approvaltests import Writer


class ExistingDirWriter(Writer):
    def __init__(self, dir_name: str) -> None:
        self.dir_name = dir_name

    def write_received_file(self, received_file: str) -> str:
        if Path(received_file).exists():
            shutil.rmtree(received_file)
        shutil.copytree(self.dir_name, received_file)
        return received_file
